#include <unistd.h>

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

#include <hipc/posix/ioutils.h>
#include <hipc/posix/usoc.h>

#include "CFGrt.h"
#include "hipcsvs_rt.h"

int hipcUsocReadReady(struct HIPC_usoc *so_p)
{
    return hipcPosixReadReady(so_p->fdr);
}

static void *runMockServer(void *arg)
{
    struct HIPC_pbank *const pbank_p = &hipcsvsRt_pbank;
    struct HIPC_queue *const queue_p = &hipcsvsRt_queue;
    struct STRUprm const *prm_p;
    struct STRUlog *log_p;
    uint8_t output;
    unsigned int tick = 0;
    unsigned int cnt = 0;

    prm_p = hipcPbankGetReadOnlyPage(pbank_p);

    for (;;) {
        fprintf(stderr, "---\n");

        if (0 == cnt) {
            if (1 == hipcPbankSwitchAccept(pbank_p)) {
                fprintf(stderr, "pbank changed\n");
                prm_p = hipcPbankGetReadOnlyPage(pbank_p);
                cnt = prm_p->size;
            }
        }

        if (0 < cnt) {
            output = prm_p->pattern[prm_p->size - cnt];
            fprintf(stderr, "tick/cnt/output %02d/%d/%d\n",
                    tick, cnt, output);

            log_p = hipcQueueFull(queue_p);
            if (NULL != log_p) {
                log_p->tick = tick;
                log_p->output = output;
                hipcQueuePush(queue_p);
            }
            --cnt;
        }
        usleep(1000 * 1000);
        ++tick;
    }

    pthread_exit(NULL);
}

int main(void)
{
    pthread_attr_t attr;
    pthread_t thread;
    struct HIPC_usoc usoc;

    hipcPosixUsocInit(&usoc, STDIN_FILENO, STDOUT_FILENO);
    hipcsvsRtInit();

    if (0 != pthread_attr_init(&attr)) {
        perror("pthread_attr_init faild");
        exit(EXIT_FAILURE);
    }
    if (0 != pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED)) {
        perror("pthread_attr_setdetachstate faild");
        exit(EXIT_FAILURE);
    }
    if (0 != pthread_create(&thread, &attr, runMockServer, NULL)) {
        perror("pthread_create faild");
        exit(EXIT_FAILURE);
    }
    if (0 != pthread_attr_destroy(&attr)) {
        perror("pthread_attr_destroy faild");
        exit(EXIT_FAILURE);
    }

    hipcMain(&usoc);

    if (0 != errno) {
        perror("Error message from perror()");
        fprintf(stderr, "errno: %d\n", errno);
    }
    fprintf(stderr, "end\n");

    return 0;
}
