#ifndef HIPC_MSGD_H
#define HIPC_MSGD_H

#include <stddef.h>

#include <hipc.h>

typedef struct HIPC_msgdata {
    unsigned char header[HIPC_MSGHDR_SIZE];
    void *body_p;
} hipc_msgd;

/**
 * Initializes a HIPC message data as a QUIT message.
 * \c errdtlstr must have both an address and a size of the string,
 * so \c errdtlstr must be a string literal or
 * a variable whose type is array of char.
 * \c errdtlstr must not be a simple pointer to char because
 * \c sizeof(char *) equals a constant value.
 *
 * @param md_p pointer to a HIPC message data
 * @param hipc_errno HIPC error number
 * @param errdtlstr error detail string
 * @returns the same value as \c hipc_errno.
 */
#define hipcGenmdQuit(md_p, hipc_errno, errdtlstr)		\
    hipcGenmdUnstru((md_p), HIPC_MESSAGE_TYPE_QUIT, (hipc_errno), (errdtlstr), sizeof(errdtlstr))

/**
 * Initializes a HIPC message data as a SUCCESS message.
 *
 * @param md_p pointer to a HIPC message data
 * @returns HIPC_SUCCESS.
 */
#define hipcGenmdSuccess(md_p)		\
    hipcGenmdUnstru((md_p), HIPC_MESSAGE_TYPE_SUCCESS, HIPC_SUCCESS, 0, 0)

/**
 * Returns a pointer to a header of a HIPC message data.
 *
 * @param md_p pointer to a HIPC message data
 * @returns pointer to the header of the HIPC message data.
 */
#define hipcMsgdHdrPtr(md_p) ((md_p)->header)

/**
 * Returns a pointer to a body of a HIPC message data.
 *
 * @param md_p pointer to a HIPC message data
 * @returns pointer to the body of the HIPC message data.
 */
#define hipcMsgdBdyPtr(md_p) ((md_p)->body_p)

#ifdef __cplusplus
extern "C" {
#endif

    extern unsigned char hipcMsgdBdySize(hipc_msgd const *const md_p);
    extern unsigned char hipcMsgdType(hipc_msgd const *const md_p);
    extern unsigned char hipcMsgdStruno(hipc_msgd const *const md_p);
    extern unsigned char hipcMsgdOffset(hipc_msgd const *const md_p);
    extern unsigned char hipcMsgdRngsize(hipc_msgd const *const md_p);
    extern enum HIPC_errno
     hipcGenmdUnstru(hipc_msgd * const md_p,
                     const unsigned char type,
                     const enum HIPC_errno hipc_errno,
                     char *const errdtlstr, size_t errdtlstr_size);
    extern enum HIPC_errno
     hipcGenmdSuccessToGet(hipc_msgd * const mds_p,
                           hipc_msgd const *const mdr_p,
                           void *const stru_p, const size_t stru_size);
    extern enum HIPC_errno
     hipcMsgdSetBdyForPut(hipc_msgd * const mdr_p,
                          void *const stru_p, const size_t stru_size);

#ifdef __cplusplus
}
#endif
#endif
