#ifndef HIPC_SESSION_H
#define HIPC_SESSION_H

#include <hipc.h>
#include <hipc/msg.h>
#include <hipc/client.h>

enum HIPC_session_state {
    HIPC_SESS_TERMINATED = 0,   /* TERMINATED is the most suitable state for uninitialized sessions. */
    HIPC_SESS_QUITTED,
    HIPC_SESS_ESTABLISHED
};

struct HIPC_session {
    struct HIPC_client cl;
    enum HIPC_session_state state;
    enum HIPC_errno (*rfnc_p) (void *, hipc_msg);
    void *rarg;
    enum HIPC_errno (*wfnc_p) (void *, const hipc_msg);
    void *warg;
};
typedef struct HIPC_session hipc_session;

#ifdef __cplusplus
extern "C" {
#endif

    extern enum HIPC_errno hipcRecvMsg(hipc_session * const sess_p,
                                       hipc_msg msgR);
    extern enum HIPC_errno hipcSendMsg(hipc_session * const sess_p,
                                       hipc_msg msgS);
    extern enum HIPC_errno hipcWaitReturn(hipc_session * const sess_p,
                                          hipc_msg msgR);

    extern enum HIPC_errno
     hipcOpen(hipc_session * const sess_p,
              struct HIPC_client const *const clbase_p,
              enum HIPC_errno (*rfnc_p) (void *, hipc_msg),
              void *rarg,
              enum HIPC_errno (*wfnc_p) (void *, const hipc_msg),
              void *warg);
    extern enum HIPC_errno
     hipcClose(hipc_session * const sess_p);
    extern enum HIPC_errno
     hipcSetHandler(hipc_session * const sess_p,
                    char const *const str,
                    void *const cimg,
                    enum HIPC_errno (*hndlr_p) (struct HIPC_client *,
                                                const hipc_msg, void *,
                                                void *), void *const arg);
    extern enum HIPC_errno
     hipcGet(hipc_session * const sess_p,
             char const *const str, void *const cimg);
    extern enum HIPC_errno
     hipcPut(hipc_session * const sess_p,
             char const *const str, void *const cimg);


    void hipcERRCHECK(hipc_session const *const sess_p,
                      char const *const filename, const int lineno);

#ifdef __cplusplus
}
#endif
/**
 * Checks whether an error has occurred.
 * See hipcERRCHECK() for more details.
 *
 * @param sess_p pointer to a HIPC session.
 * @returns pointer to the body of the HIPC message data.
 */
#define HIPC_ERRCHECK(sess_p) hipcERRCHECK(sess_p, __FILE__, __LINE__)
#endif
