#include <errno.h>

#include <hipc/posix/ioutils.h>
#include <hipc/posix/clientutils.h>

/**
 * Read a HIPC message from \c fd.
 * \c errno is set to 0 at the beginning of this function.
 *
 * @param arg A pointer to a \c fd
 * @param msg A HIPC message is read to this argument.
 * @retval HIPC_SUCCESS A message is read successfully.
 * @retval HIPC_ERR_IO_READ means occurrence of an error related to IO.
 *         \c errno may be set by lower layers.
 */
enum HIPC_errno hipcPosixReadMsg(void *arg, hipc_msg msg)
{
    int const *const fd_p = arg;
    ssize_t i;

    errno = 0;
    i = hipcPosixReadExactly(*fd_p, msg, HIPC_MSGHDR_SIZE);
    if (0 >= i) {
        return HIPC_ERR_IO_READ;
    }
    if (0 < hipcMsgBdySize(msg)) {
        i = hipcPosixReadExactly(*fd_p, &msg[HIPC_MSGHDR_SIZE],
                                 hipcMsgBdySize(msg));
        if (0 >= i) {
            return HIPC_ERR_IO_READ;
        }
    }

    return HIPC_SUCCESS;
}

/**
 * Write a HIPC message to \c fd.
 * \c errno is set to 0 at the beginning of this function.
 *
 * @param arg A pointer to a \c fd
 * @param msg A HIPC message to be written
 * @retval HIPC_SUCCESS A message is written successfully.
 * @retval HIPC_ERR_IO_WRITE means occurrence of an error related to IO.
 *         \c errno may be set by lower layers.
 */
enum HIPC_errno hipcPosixWriteMsg(void *arg, const hipc_msg msg)
{
    int const *const fd_p = arg;

    errno = 0;
    if (-1 == hipcPosixWriteExactly(*fd_p, msg, hipcMsgTotalSize(msg))) {
        return HIPC_ERR_IO_WRITE;
    }

    return HIPC_SUCCESS;
}
