#include <stddef.h>

#include <hipc.h>
#include <hipc/posix/ioutils.h>
#include <hipc/posix/usoc.h>

/**
 * Initializes a universal socket.
 *
 * @param so_p Pointer to a universal socket
 * @param fdr a file descriptor for reading
 * @param fdw a file descriptor for writing
 */
void hipcPosixUsocInit(struct HIPC_usoc *so_p, const int fdr,
                       const int fdw)
{
    so_p->fdr = fdr;
    so_p->fdw = fdw;

    return;
}

enum HIPC_errno
hipcUsocRead(struct HIPC_usoc *so_p, void *buf, const size_t size)
{
    if (0 < size) {
        if (0 >= hipcPosixReadExactly(so_p->fdr, buf, size)) {
            return HIPC_ERR_S_IO_READ;
        }
    }

    return HIPC_SUCCESS;
}

enum HIPC_errno
hipcUsocWrite(struct HIPC_usoc *so_p, const void *buf, const size_t size)
{
    if (-1 == hipcPosixWriteExactly(so_p->fdw, buf, size)) {
        return HIPC_ERR_S_IO_WRITE;
    }

    return HIPC_SUCCESS;
}
