#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>

#include "client.c"

static void test_StdErrno(void)
{
    struct HIPC_client cl;
    int errno_saved;

    errno_saved = errno;

    errno = 0;
    hipcClientSetStdErrno(&cl);
    assert(0 == hipcClientGetStdErrno(&cl));

    errno = 1;
    hipcClientSetStdErrno(&cl);
    assert(1 == hipcClientGetStdErrno(&cl));

    errno = errno_saved;

    return;
}

static void test_ErrorDetailStr(void)
{
    struct HIPC_client cl;
    const char *str;

    hipcClientSetErrDtlStr(&cl, "012", NULL, 0);
    str = hipcClientGetErrDtlStr(&cl);
    assert(0 == strcmp("012", str));

    hipcClientSetErrDtlStr(&cl, "012", NULL, 9);
    str = hipcClientGetErrDtlStr(&cl);
    assert(0 == strcmp("012", str));

    hipcClientSetErrDtlStr(&cl, "abc", "", 0);
    str = hipcClientGetErrDtlStr(&cl);
    assert(0 == strcmp("abc", str));

    hipcClientSetErrDtlStr(&cl, "012", "345", 3);
    str = hipcClientGetErrDtlStr(&cl);
    assert(0 == strcmp("012345", str));

    hipcClientSetErrDtlStr(&cl, "012", "345", 2);
    str = hipcClientGetErrDtlStr(&cl);
    assert(0 == strcmp("01234", str));

    hipcClientSetErrDtlStr(&cl, "012", "345", 1);
    str = hipcClientGetErrDtlStr(&cl);
    assert(0 == strcmp("0123", str));

    hipcClientSetErrDtlStr(&cl, "012", "345", 0);
    str = hipcClientGetErrDtlStr(&cl);
    assert(0 == strcmp("012345", str));

    hipcClientSetErrDtlStr(&cl, NULL, "xxx", 3);
    str = hipcClientGetErrDtlStr(&cl);
    assert(0 == strcmp("012345", str));

    hipcClientSetErrDtlStr(&cl, NULL, NULL, 0);
    str = hipcClientGetErrDtlStr(&cl);
    assert(0 == strcmp("012345", str));

    return;
}

static void test_ErrorDetailStr_longStr(void)
{
    static char buf[HIPC_IMG_ALLOCSIZE + 2];
    struct HIPC_client cl;
    const char *str;
    size_t i;

    for (i = 0; i < sizeof(buf); ++i) {
        buf[i] = 'a';
    }
    buf[HIPC_IMG_ALLOCSIZE - 2] = 'x';
    buf[HIPC_IMG_ALLOCSIZE - 1] = 'y';
    buf[HIPC_IMG_ALLOCSIZE] = 'z';
    buf[HIPC_IMG_ALLOCSIZE + 1] = '\0';
    hipcClientSetErrDtlStr(&cl, buf, "AAA", 0);
    str = hipcClientGetErrDtlStr(&cl);
    for (i = 0; i < HIPC_IMG_ALLOCSIZE - 2; ++i) {
        assert('a' == str[i]);
    }
    assert('x' == str[HIPC_IMG_ALLOCSIZE - 2]);
    assert('\0' == str[HIPC_IMG_ALLOCSIZE - 1]);

    buf[HIPC_IMG_ALLOCSIZE] = '\0';
    hipcClientSetErrDtlStr(&cl, buf, NULL, 0);
    str = hipcClientGetErrDtlStr(&cl);
    for (i = 0; i < HIPC_IMG_ALLOCSIZE - 2; ++i) {
        assert('a' == str[i]);
    }
    assert('x' == str[HIPC_IMG_ALLOCSIZE - 2]);
    assert('\0' == str[HIPC_IMG_ALLOCSIZE - 1]);

    buf[HIPC_IMG_ALLOCSIZE - 1] = '\0';
    hipcClientSetErrDtlStr(&cl, buf, NULL, 0);
    str = hipcClientGetErrDtlStr(&cl);
    for (i = 0; i < HIPC_IMG_ALLOCSIZE - 2; ++i) {
        assert('a' == str[i]);
    }
    assert('x' == str[HIPC_IMG_ALLOCSIZE - 2]);
    assert('\0' == str[HIPC_IMG_ALLOCSIZE - 1]);

    hipcClientSetErrDtlStr(&cl, buf, "", 0);
    str = hipcClientGetErrDtlStr(&cl);
    for (i = 0; i < HIPC_IMG_ALLOCSIZE - 2; ++i) {
        assert('a' == str[i]);
    }
    assert('x' == str[HIPC_IMG_ALLOCSIZE - 2]);
    assert('\0' == str[HIPC_IMG_ALLOCSIZE - 1]);

    hipcClientSetErrDtlStr(&cl, buf, "A", 0);
    str = hipcClientGetErrDtlStr(&cl);
    for (i = 0; i < HIPC_IMG_ALLOCSIZE - 2; ++i) {
        assert('a' == str[i]);
    }
    assert('x' == str[HIPC_IMG_ALLOCSIZE - 2]);
    assert('\0' == str[HIPC_IMG_ALLOCSIZE - 1]);

    hipcClientSetErrDtlStr(&cl, buf, "AA", 0);
    str = hipcClientGetErrDtlStr(&cl);
    for (i = 0; i < HIPC_IMG_ALLOCSIZE - 2; ++i) {
        assert('a' == str[i]);
    }
    assert('x' == str[HIPC_IMG_ALLOCSIZE - 2]);
    assert('\0' == str[HIPC_IMG_ALLOCSIZE - 1]);

    buf[HIPC_IMG_ALLOCSIZE - 2] = '\0';
    hipcClientSetErrDtlStr(&cl, buf, "", 0);
    str = hipcClientGetErrDtlStr(&cl);
    for (i = 0; i < HIPC_IMG_ALLOCSIZE - 2; ++i) {
        assert('a' == str[i]);
    }
    assert('\0' == str[HIPC_IMG_ALLOCSIZE - 2]);
    assert('\0' == str[HIPC_IMG_ALLOCSIZE - 1]);

    hipcClientSetErrDtlStr(&cl, buf, "A", 0);
    str = hipcClientGetErrDtlStr(&cl);
    for (i = 0; i < HIPC_IMG_ALLOCSIZE - 2; ++i) {
        assert('a' == str[i]);
    }
    assert('A' == str[HIPC_IMG_ALLOCSIZE - 2]);
    assert('\0' == str[HIPC_IMG_ALLOCSIZE - 1]);

    hipcClientSetErrDtlStr(&cl, buf, "BB", 0);
    str = hipcClientGetErrDtlStr(&cl);
    for (i = 0; i < HIPC_IMG_ALLOCSIZE - 2; ++i) {
        assert('a' == str[i]);
    }
    assert('B' == str[HIPC_IMG_ALLOCSIZE - 2]);
    assert('\0' == str[HIPC_IMG_ALLOCSIZE - 1]);

    return;
}

static void test_Err(void)
{
    struct HIPC_client cl;
    const char *str;

    hipcClientSetError(&cl, HIPC_ERR_PROTOCOL_VIOLATION, "NG");
    assert(HIPC_ERR_PROTOCOL_VIOLATION == hipcClientGetHipcErrno(&cl));
    str = hipcClientGetErrStr(&cl);
    assert(0 ==
           strcmp("HIPC_ERR_PROTOCOL_VIOLATION: Protocol violation", str));
    str = hipcClientGetErrDtlStr(&cl);
    assert(0 == strcmp("NG", str));

    hipcClientSetError(&cl, HIPC_SUCCESS, "");
    assert(HIPC_SUCCESS == hipcClientGetHipcErrno(&cl));
    str = hipcClientGetErrStr(&cl);
    assert(0 == strcmp("HIPC_SUCCESS: No error", str));
    str = hipcClientGetErrDtlStr(&cl);
    assert(0 == strcmp("", str));

    hipcClientSetError(&cl, HIPC_N_ERR, "");
    str = hipcClientGetErrStr(&cl);
    assert(0 == strcmp("Unknown HIPC errorno value", str));

    return;
}

int main(int argc, char *argv[])
{
    if (2 != argc) {
        fprintf(stderr, "Usage: %s test_id\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    switch (argv[1][0]) {
    case 'a':
        test_StdErrno();
        break;
    case 'b':
        test_ErrorDetailStr();
        break;
    case 'c':
        test_ErrorDetailStr_longStr();
        break;
    case 'd':
        test_Err();
        break;
    default:
        fprintf(stderr, "Invalid test_id: %s \n", argv[1]);
        exit(EXIT_FAILURE);
    }

    return 0;
}
