#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <hipc/client.h>

static void test_hipcGenmsgBye(void)
{
    hipc_msg msg;

    hipcGenmsgBye(msg);
    assert(HIPC_MESSAGE_TYPE_BYE == msg[HIPC_MSGHDR_BYTE_TYPE]);
    assert(0 == msg[HIPC_MSGHDR_BYTE_OFFSET]);
    assert(HIPC_UNSTRU == msg[HIPC_MSGHDR_BYTE_STRUNO]);
    assert(0 == msg[HIPC_MSGHDR_BYTE_BDYSZ]);

    return;
}

const struct HIPC_cl_indp HIPCUD_tclmsg_indp = {
    (unsigned char *) "id", 2, 5, 2, NULL
};

unsigned char HIPCUD_tclmsg_structsizes[] = {
    4,
    8,
};
struct HIPC_cl_dp HIPCUD_tclmsg_smdp =
    { '\0', HIPCUD_tclmsg_structsizes, NULL };
unsigned char HIPCUD_tclmsg_simg_buf0[4];
unsigned char HIPCUD_tclmsg_simg_buf1[8];
unsigned char *HIPCUD_tclmsg_simg[] = {
    HIPCUD_tclmsg_simg_buf0,
    HIPCUD_tclmsg_simg_buf1,
};

const struct HIPC_client cl_msg = {
    0,
    &HIPCUD_tclmsg_indp,
    NULL,
    &HIPCUD_tclmsg_smdp,
    HIPCUD_tclmsg_simg,
};

static void test_hipcClientCheckStrutpl(void)
{
    struct HIPC_client cl = cl_msg;

    /* STRUNO */
    assert(HIPC_ERR_PROTOCOL_VIOLATION ==
           hipcClientCheckStrutpl(&cl, 2, 0, 0));
    assert(HIPC_SUCCESS == hipcClientCheckStrutpl(&cl, 1, 0, 0));
    assert(HIPC_SUCCESS == hipcClientCheckStrutpl(&cl, 0, 0, 0));

    /* INV_OFFSET */
    assert(HIPC_ERR_PROTOCOL_VIOLATION ==
           hipcClientCheckStrutpl(&cl, 1, 8, 0));
    assert(HIPC_SUCCESS == hipcClientCheckStrutpl(&cl, 1, 7, 0));
    assert(HIPC_ERR_PROTOCOL_VIOLATION ==
           hipcClientCheckStrutpl(&cl, 0, 4, 0));
    assert(HIPC_SUCCESS == hipcClientCheckStrutpl(&cl, 0, 3, 0));

    /* INV_SIZE */
    assert(HIPC_ERR_PROTOCOL_VIOLATION ==
           hipcClientCheckStrutpl(&cl, 1, 0, 9));
    assert(HIPC_SUCCESS == hipcClientCheckStrutpl(&cl, 1, 0, 8));
    assert(HIPC_ERR_PROTOCOL_VIOLATION ==
           hipcClientCheckStrutpl(&cl, 0, 0, 5));
    assert(HIPC_SUCCESS == hipcClientCheckStrutpl(&cl, 0, 0, 4));

    /* OFFSET_SIZE */
    assert(HIPC_ERR_PROTOCOL_VIOLATION ==
           hipcClientCheckStrutpl(&cl, 1, 2, 7));
    assert(HIPC_SUCCESS == hipcClientCheckStrutpl(&cl, 1, 2, 6));
    assert(HIPC_ERR_PROTOCOL_VIOLATION ==
           hipcClientCheckStrutpl(&cl, 0, 1, 4));
    assert(HIPC_SUCCESS == hipcClientCheckStrutpl(&cl, 0, 1, 3));

    return;
}

static void test_hipcGenmsgGet(void)
{
    struct HIPC_client cl = cl_msg;
    hipc_msg msg;

    assert(HIPC_ERR_PROTOCOL_VIOLATION ==
           hipcGenmsgGet(msg, &cl, 2, 0, 0));

    assert(HIPC_SUCCESS == hipcGenmsgGet(msg, &cl, 0, 1, 2));
    assert(HIPC_MESSAGE_TYPE_GET == msg[HIPC_MSGHDR_BYTE_TYPE]);
    assert(0 == msg[HIPC_MSGHDR_BYTE_STRUNO]);
    assert(1 == msg[HIPC_MSGHDR_BYTE_OFFSET]);
    assert(2 == msg[HIPC_MSGHDR_BYTE_BDYSZ]);

    return;
}

static void test_hipcGenmsgPut(void)
{
    struct HIPC_client cl = cl_msg;
    hipc_msg msg;

    assert(HIPC_ERR_PROTOCOL_VIOLATION ==
           hipcGenmsgPut(msg, &cl, 2, 0, 0));

    assert(HIPC_SUCCESS == hipcGenmsgPut(msg, &cl, 0, 1, 2));
    assert(HIPC_MESSAGE_TYPE_PUT == msg[HIPC_MSGHDR_BYTE_TYPE]);
    assert(0 == msg[HIPC_MSGHDR_BYTE_STRUNO]);
    assert(1 == msg[HIPC_MSGHDR_BYTE_OFFSET]);
    assert(2 == msg[HIPC_MSGHDR_BYTE_BDYSZ]);

    return;
}

static void test_hipcGenmsgHello(void)
{
    struct HIPC_client cl = cl_msg;
    hipc_msg msg;

    hipcGenmsgHello(msg, &cl);
    assert(HIPC_MESSAGE_TYPE_HELLO == msg[HIPC_MSGHDR_BYTE_TYPE]);
    assert(HIPC_UNSTRU == msg[HIPC_MSGHDR_BYTE_STRUNO]);
    assert(0 == msg[HIPC_MSGHDR_BYTE_OFFSET]);
    assert(2 == msg[HIPC_MSGHDR_BYTE_BDYSZ]);
    assert(0 == memcmp(&msg[HIPC_MSGHDR_SIZE], "id", 2));

    return;
}

int main(int argc, char *argv[])
{
    if (2 != argc) {
        fprintf(stderr, "Usage: %s test_id\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    switch (argv[1][0]) {
    case 'a':
        test_hipcGenmsgBye();
        break;
    case 'b':
        test_hipcClientCheckStrutpl();
        break;
    case 'c':
        test_hipcGenmsgGet();
        break;
    case 'd':
        test_hipcGenmsgPut();
        break;
    case 'e':
        test_hipcGenmsgHello();
        break;
    default:
        fprintf(stderr, "Invalid test_id: %s \n", argv[1]);
        exit(EXIT_FAILURE);
    }

    return 0;
}
