#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include <hipc/msg.h>

static void test_msg_basics(void)
{
    assert(HIPC_MSG_ALLOCSIZE == sizeof(hipc_msg));
    return;
}

static void test_msg_sizes(void)
{
    hipc_msg msg;

    msg[HIPC_MSGHDR_BYTE_TYPE] = HIPC_MESSAGE_TYPE_GET;
    msg[HIPC_MSGHDR_BYTE_BDYSZ] = 0;
    assert(0 == hipcMsgRngsize(msg));
    assert(0 == hipcMsgBdySize(msg));
    assert(HIPC_MSGHDR_SIZE == hipcMsgTotalSize(msg));

    msg[HIPC_MSGHDR_BYTE_TYPE] = HIPC_MESSAGE_TYPE_GET;
    msg[HIPC_MSGHDR_BYTE_BDYSZ] = 3;
    assert(3 == hipcMsgRngsize(msg));
    assert(0 == hipcMsgBdySize(msg));
    assert(HIPC_MSGHDR_SIZE == hipcMsgTotalSize(msg));

    msg[HIPC_MSGHDR_BYTE_TYPE] = HIPC_MESSAGE_TYPE_GET;
    msg[HIPC_MSGHDR_BYTE_BDYSZ] = 255;
    assert(255 == hipcMsgRngsize(msg));
    assert(0 == hipcMsgBdySize(msg));
    assert(HIPC_MSGHDR_SIZE == hipcMsgTotalSize(msg));

    msg[HIPC_MSGHDR_BYTE_TYPE] = HIPC_MESSAGE_TYPE_PUT;
    msg[HIPC_MSGHDR_BYTE_BDYSZ] = 0;
    assert(0 == hipcMsgRngsize(msg));
    assert(0 == hipcMsgBdySize(msg));
    assert(HIPC_MSGHDR_SIZE == hipcMsgTotalSize(msg));

    msg[HIPC_MSGHDR_BYTE_TYPE] = HIPC_MESSAGE_TYPE_PUT;
    msg[HIPC_MSGHDR_BYTE_BDYSZ] = 3;
    assert(3 == hipcMsgRngsize(msg));
    assert(3 == hipcMsgBdySize(msg));
    assert((3 + HIPC_MSGHDR_SIZE) == hipcMsgTotalSize(msg));

    msg[HIPC_MSGHDR_BYTE_TYPE] = HIPC_MESSAGE_TYPE_PUT;
    msg[HIPC_MSGHDR_BYTE_BDYSZ] = 255;
    assert(255 == hipcMsgRngsize(msg));
    assert(255 == hipcMsgBdySize(msg));
    assert((255 + HIPC_MSGHDR_SIZE) == hipcMsgTotalSize(msg));

    msg[HIPC_MSGHDR_BYTE_TYPE] = HIPC_MESSAGE_TYPE_SYS;
    msg[HIPC_MSGHDR_BYTE_BDYSZ] = 5;
    assert(5 == hipcMsgRngsize(msg));
    assert(5 == hipcMsgBdySize(msg));
    assert((5 + HIPC_MSGHDR_SIZE) == hipcMsgTotalSize(msg));

    msg[HIPC_MSGHDR_BYTE_TYPE] = 100;
    msg[HIPC_MSGHDR_BYTE_BDYSZ] = 7;
    assert(7 == hipcMsgRngsize(msg));
    assert(7 == hipcMsgBdySize(msg));
    assert((7 + HIPC_MSGHDR_SIZE) == hipcMsgTotalSize(msg));

    return;
}

static void test_hipcMsgType(void)
{
    hipc_msg msg;

    msg[HIPC_MSGHDR_BYTE_TYPE] = 11;
    assert(11 == hipcMsgType(msg));

    return;
}

static void test_hipcMsgStruno(void)
{
    hipc_msg msg;

    msg[HIPC_MSGHDR_BYTE_STRUNO] = 127;
    assert(127 == hipcMsgStruno(msg));

    return;
}

static void test_hipcMsgOffset(void)
{
    hipc_msg msg;

    msg[HIPC_MSGHDR_BYTE_OFFSET] = 8;
    assert(8 == hipcMsgOffset(msg));

    return;
}

static void test_hipcMsgBdyPtr(void)
{
    hipc_msg msg;

    msg[HIPC_MSGHDR_SIZE] = 0;
    assert(0 == *(unsigned char *) hipcMsgBdyPtr(msg));

    msg[HIPC_MSGHDR_SIZE] = 19;
    assert(19 == *(unsigned char *) hipcMsgBdyPtr(msg));

    return;
}

int main(int argc, char *argv[])
{
    if (2 != argc) {
        fprintf(stderr, "Usage: %s test_id\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    switch (argv[1][0]) {
    case 'a':
        test_msg_basics();
        break;
    case 'b':
        test_msg_sizes();
        break;
    case 'c':
        test_hipcMsgType();
        break;
    case 'd':
        test_hipcMsgStruno();
        break;
    case 'e':
        test_hipcMsgOffset();
        break;
    case 'f':
        test_hipcMsgBdyPtr();
        break;
    default:
        fprintf(stderr, "Invalid test_id: %s \n", argv[1]);
        exit(EXIT_FAILURE);
    }

    return 0;
}
