#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include <hipc/scfg.h>

static void test_hipcScfgInit(void)
{
    struct HIPC_scfg cfg;
    unsigned char endian;

    cfg.endian_p = &endian;
    hipcScfgInit(&cfg);
    assert(0x6c /* 'l' */  == endian ||
           0x42 /* 'B' */  == endian ||
           0x6f /* 'o' */  == endian);

    return;
}

static void test_hipcScfgCheckStrusize(void)
{
    struct HIPC_scfg cfg;
    const size_t sizes1[] = { 0, 1, 8, 255 };
    const size_t sizes2[] = { 16, 256, 32 };

    cfg.strusize_p = NULL;
    assert(HIPC_SUCCESS == hipcScfgCheckStrusize(&cfg));

    cfg.n_struct = sizeof(sizes1) / sizeof(sizes1[0]);
    cfg.strusize_p = sizes1;
    assert(HIPC_SUCCESS == hipcScfgCheckStrusize(&cfg));

    cfg.n_struct = sizeof(sizes2) / sizeof(sizes2[0]);
    cfg.strusize_p = sizes2;
    assert(HIPC_ERR_S_OVERSIZED_STRU == hipcScfgCheckStrusize(&cfg));

    return;
}

int main(int argc, char *argv[])
{
    if (2 != argc) {
        fprintf(stderr, "Usage: %s test_id\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    switch (argv[1][0]) {
    case 'a':
        test_hipcScfgInit();
        break;
    case 'b':
        test_hipcScfgCheckStrusize();
        break;
    default:
        fprintf(stderr, "Invalid test_id: %s \n", argv[1]);
        exit(EXIT_FAILURE);
    }

    return 0;
}
